/* $Id: drawxtl.h 616 2007-01-12 19:39:27Z Larry $ */

#define OPENGL 1                     /* define to include openGL code */
#define VERSION "4.3"
#define MAX_ATOM 1000				 /* Maximum number of atoms in asymmetric unit */
#define MAX_EL 200                   /* Maximum number of anisotropic atoms */
#define MAX_POLY 30                  /* Maximum number of polyhedral types */
#define MAX_PLANE 30				 /* Maximum number of plane types */
#define MAX_BOND 50                  /* Maximum number of bond types */
#define MAX_CONE 10                  /* Maximum number of lone pair cone types */
#define MAX_SPHERE 30				 /* Maximum number of sphere types */
#define RAD 57.2957795				 /* Conversion from radians to degrees */
#define MAX_VERTS 20000				 /* Maximum number of generated vertices */
#define MAX_LABELS 1000				 /* Maximum number of textual labels */
#define MAX_ARROW 30                 /* maximum number of arrows */
#define MAX_FRAME 15                 /* Maximum number of frames */
#define MAX_MAP_CONTOUR 10           /* Maximum number of map contours */
#define MAX_BPLANE 10                /* Maximum number of best planes */
#define MAX_MODULATION 20            /* Maximum number of modulation vectors */
#ifndef PI
#define PI 3.1415926
#endif
#define O_OPT 1
#define B_OPT 2
#define P_OPT 4
#define V_OPT 8
#define M_OPT 16
#define L_OPT 32
#define BUFSIZE 512
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#define EPS 1e-7
#define SPIN 1
#define ZOOM 2
#define PAN 3
#define DRAG 4

typedef struct {
	float s;
	float v[3];
} QUAT;

typedef float XY[2];

typedef float XYZ[3];

#ifdef OPENGL
#if defined(__APPLE__) && defined(__MACH__) 
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#  include <GL/gl.h>
#endif
# if defined(FREEGLUT)
#  include <GL/freeglut_ext.h>
# endif 
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#ifdef WIN32
  #include <direct.h>
#endif


#ifndef max
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define min(a,b) ((a) <= (b) ? (a) : (b))
#endif

/* prototype definitions */

void add_to_list (float *, int);
void add_vert (float *, int, int, int);
void add_vert_nc (float *);
void analyze_bonds (void);
void axeqb (double a[3][3], double b[3], double c[3]);
void axis_to_quaternion(float *axis, float theta, QUAT *quat);
void Blank_Strip (char *);
void build_box_contents (void);
void build_poly_list (int, int, int, int, float *);
void Calc_Rot (float *, float *);
int  check_atom_name(char *, char *);
void Conv_Sym_Mat (void);
void convert_ellipsoid (void);
float convert_pos(char *);
void crystal_compute (int, int);
void delay (int);
float determinant (double x[3][3]);
float dist (int, int);
void do_drawing (void);
float dot0_3d(float,float,float,float,float,float,float,float,float);
void draw_cell (int);
void draw_GL_triple(void);
void draw_cursor (void);            /* display openGL cursor */
void Dump_View(void);
void expand_atom (int);
int  eigen (float *, float *, float *, float a[3][3]);
void Error_Box(char *);
int fillcube(unsigned char *, FILE *);
int find_atom(void);
int find_proj_atom(int, int);
void find_all_in_box (int);
void find_lattice_type( void);
void generate_arrows (void);
void generate_bonds (void);
void generate_cones (void);
void generate_ellipsoids (void);
void generate_gl_texts(void);
void generate_lsq_planes(void);
void generate_planes (void);
void generate_poly (void);
void generate_slab(void);
void generate_spheres (void);
void generate_texts (void);
void generate_triple(void);
void get_atom_id (void);
void get_input (int);
void get_label (char *, char *, char *, char *, char *, int);
int  get_next_token(char *, int, FILE *);
void import_cif (char *, int, int, int *, int);
void import_fdat (char *, int, int, int);
void import_gsas (char *, int, int, int);
void import_pcr (char *, int, int, int);
void import_schakal (char *, int, int, int);
void import_shelx (char *, int, int, int);
void import_wien (char *, int, int, int);
void import_discus (char *, int, int, int);
void label_cell(void);
void Locate_Triple (void);
void make_bmat (int sys, float lat_con[6], double b_mat[3][3], float ginv[3][3],
                float rec_lat_con[6]);
float matinv(float a[3][3]);
int  Maximize_rho(int);
void modulate_parameters(float vert[3], double *occ, int sym_no, int atom_no);
int modulate_uij(float vert[3], int ellips_no, int atom_no, int sym_no);
void move_cursor(int, float);
int  not_in_slab(float,float,float);
float P_to_C (float);
void Output_Spheres (float *, int);
void plot_vrml_poly (int);
float *polygon_normal_3d(int, float*);
float polygon_solid_angle_3d(int, float*, float*);
int position_cif(int, FILE *, char *, char *, int);
void print_map_info(int);
void print_sym (void);
void Progress_Window(int, char *, float);
void push_cylinder (float df[3], float at[3], float radius, char * color);
void quaternion_to_rotmatrix(QUAT *q, float *m);
void qmult(QUAT *q1, QUAT *q2, QUAT *dest);
void qnormalize(QUAT *q);
void read_dn6(char * infile, int);
void read_fcf(char * infile, int);
void read_m80(char * infile, int);
void read_flp(char * infile, int);
void read_grd(char * infile, int);
void read_inp (int);
void read_stf(char * infile, int);
void read_vasp(char * infile, int);
void read_w2k(char * infile, int);
void read_exc(char * infile, int);
void sub_add_vert(float * vert, int no_cell);
void sub_add_vert_nc(float * vert, int no_cell);
void symop (char *);
void show_slab(void);
void skip_blocks(int i, FILE * in);
void tb_init(void);
float tb_project_to_sphere(float, float, float);
void Token_Strip (char *, int);
int toupper (int);
void trackball(void);
void Transform_POV_Color(char *);
void Transform_VRML_Color(char *);
void trim_string(char string[], int len);
int Unique_Atom (void);
void Update_Cursor_List(float *, int);
void usage (void);
void vadd(float *src1, float *src2, float *dst);
float vdot(float *v1, float *v2);
void vcross(float *v1, float *v2, float *cross);
void vcopy(float *vsrc, float *vdst);
int vec_dif(int n1, float v1[3], int n2, float v2[3], int n3, float v3[3], float v[3]);
float vlength(float *v);
void vmult(float *src1, float *src2, float *dst);
float *vnew(float x, float y, float z);
void vnormalize(float *v);
void vprint(float *v);
void vscale(float *v, float f);
void vset(float *v, float x, float y, float z);
void vsub(float *src1, float *src2, float *dst);
void vzero(float *v);
void window_to_tb(float mx, float my, float *x, float *y);
QUAT XYZ_Rot_to_Q(float xrot, float yrot, float zrot);
void *zalloc(size_t size);
int getsys(int);
void findsys(void);
void getsym(char*, int, int);

typedef struct
{
    int info_valid;
    int map_type;
    char title[82];
    float lat_con[6];
    float rhomx;
    float rhomn;
    int map_int[3];
    float xlim[2];
    float ylim[2];
    float zlim[2];
}  MAP_INFO;

typedef struct
{
/* global variables for lattice constants and symmetry operators */

    char spg[3][4];	        /* space group code */
    int ss[24][3][3];           /* rotation operators for symmetry */
    float rss[24][3][3];        /* unitary rotation operators for symmetry */
    float lat_pos[4][3];        /* coordinates of lattice points */
    float ts[24][3];            /* translation operators */
    int sym_op_no[192];         /* saved symmetry operation number */
    float cell_xyz[192][3];     /* storage for generated equiv. positions */
    int sys;                    /* crystal system indicator */
    int nbr;                    /* Bravais lattice indicator */
    int acentric;               /* true in non-centric space group */
    int ng;	                /* number of rotation operations to specify group */
    int nlat;                   /* number of lattice positions */
    int origin1_flag;           /* flag for origin1 space group */
    float origin_offset[3];
    float label_scale;
    char col_cell[40];          /* color and properties of framebars */

/* global variables for file names and file variables */

    char savedpath[100];        /* path at start of program */
    char File_Root[100];        /* Input File Name with extension stripped */
    FILE *fpin, *flout;
    FILE *fpoutv;
    FILE *fpoutp;
    char *Str_File_Inp;         /* Input File Name */
    char *Str_File;
    char Pov_File[100];
    char *output;

/* global variables for atom lists */

    int *atom_no;               /* master list atom and symmetry number */
    int *orig_atom_no;          /* saved atom number */
    float atom_xyz[MAX_ATOM][3];/* atomic coordinates */
    char atom_l[MAX_ATOM][5];   /* atom labels */
    int atom_n[MAX_ATOM];       /* input atom number */
    int sv_atom_n[MAX_ATOM];    /* saved input atom number */
    float xyzoff[3];            /* amount to be added to each atom position */
    int atom_fn[MAX_ATOM];      /* frame number for atom */

/* atom list variables for modulated structures */

  int modulated;             /* flag for modulated or conventional/average structure */
  int no_mod_vectors;        /* number of atom_site_fourier_wave_vectors */
  int no_site_displace;      /* number of atom_site_displace_fourier terms */
  int no_site_U_terms;       /* number of atom_site_U_fourier_terms */
  int no_site_occ;           /* number of atom_site_occ_fourier terms */
  int no_cell_vec;           /* number of cell_wave_vectors */
  float cell_vec[3][3];      /* cell_wave_vectors */
  int no_subsys;             /* number of subsystems */
  float subsys_ref_volume;   /* reference reciprocal volume */
  float subsys_vol[10];      /* reciprocal volume for each of the subsystems */
  float subsys_fact[10][3][3]; /* place to save relationship between main and subsystem cells */
  int   vector_mult[MAX_MODULATION][3]; /* coefficients ijk for modvector = i*cell_vec_1 + j*cell_vec_2 ..*/
  float modvector[MAX_MODULATION][3];   /* atom Fourier modulation vectors */
  float atom_modpar[3*MAX_ATOM*MAX_MODULATION][2]; /* cosine and sine multiplier for an item */
  float atom_mod_sawtooth[MAX_ATOM*MAX_MODULATION][5]; /* amplitudes and interval for sawtooth modulation */
  int   atom_modpar_atom[3*MAX_ATOM*MAX_MODULATION]; /* atom number for atom_modpar */
  int   atom_modpar_axis[3*MAX_ATOM*MAX_MODULATION]; /* axis number (0-2) for atom_modpar */
  int   atom_modpar_id[3*MAX_ATOM*MAX_MODULATION]; /* entry in vector_mult for this term (1 - no_mod_vectors) */
  float atom_occpar[MAX_ATOM*MAX_MODULATION][2]; /* cosine and sine multiplier of occ fourier series */
  int   atom_occpar_atom[MAX_ATOM*MAX_MODULATION]; /* atom number for atom_occpar */
  int   atom_occpar_id[MAX_ATOM*MAX_MODULATION]; /* axis number (0-2) for atom_occpar */
  float atom_occ_crenel[MAX_ATOM*MAX_MODULATION][2];   /* crenel lower and upper limit for occupancy */
  int   atom_ismod[MAX_ATOM];                   /* flag for modulated position */
  int   occ_ismod[MAX_ATOM];                    /* flag for modulated occupancy */
  float occupancy[MAX_ATOM];                    /* average occupancy */
  float min_occ[MAX_ATOM];                      /* minimum occupancy required for drawing the atom */
  int   ellips_ismod[MAX_EL];                   /* flag for modulated Uij */
  float ellips_modpar[3*MAX_EL*MAX_MODULATION][2];  /* wave vector id, cosine and sine multip */
  int   ellips_modpar_id[3*MAX_EL*MAX_MODULATION];  /* entry for this term (1 - no_mod_vectors) */
  int   ellips_modpar_term[3*MAX_EL*MAX_MODULATION]; /* identity of this term (1-6); */
  int   ellips_modpar_atom[MAX_ATOM*MAX_MODULATION]; /* atom number for this item */
  float phaseshift[3];                         /* modulation phase offset t for each modulation */
  float * vert_occ;                            /* occupancy at individual sites (for radius scaling) */
  float ts_m[24][3];                           /* translational component of superspace symmetry */
  int ss_m[24][3][3];                          /* rotational component of superspace symmetry */

/* cone parameters */

   int ncone;                      /* number of different types of lonepair cones */
   char cone_l1[MAX_CONE][4];      /* label for one end of bond */
   int numlonepairs[MAX_CONE];     /* number of lone pairs to draw at atom */
   float cone_height[MAX_CONE];    /* maximum lone pair distance */
   float cone_min[MAX_CONE];       /* cone tip radius */
   float cone_max[MAX_CONE];       /* cone base radius */
   char col_cone[MAX_CONE][40];    /* color and properties of cones */
   int cone_fn[MAX_CONE];          /* frame number for cone */

/* magnetic arrow parameters */

  int nmag;                       /* number of magnetic moments */
  int arrow_fn[MAX_ARROW];        /* saved frame number */
  char col_arrow[MAX_ARROW][40];  /* arrow color */
  float arrow_length[MAX_ARROW];  /* length of mag. arrow */
  float arrow_diam[MAX_ARROW];    /* diameter of magnetic arrow */
  float mag_matrix[3][3];         /* transformation between magnetic and nuclear cell */
  float mag_xp[MAX_ARROW][3];     /* position of arrow in real-space cell */
  float mag_xc[MAX_ARROW][3];     /* components of arrow wrt XYZ */

/* polyhedra parameters */

  int npoly;                  /* number of different types of polyhedra */
  int poly_fn[MAX_POLY];      /* frame number for polyhedra commands */
  float polylimit;            /* should be close to zero to allow only regular polygonal faces */
  char poly_l[MAX_POLY][5];   /* polyhedral center label */
  char poly_t[MAX_POLY][5];   /* polyhedral vertex label */
  float poly_size[MAX_POLY];  /* size of polyhedron */
  float poly_min[MAX_POLY];   /* lower limit for shell-like polyhedron */
  char poly_col[MAX_POLY][40];/* polyhedron colors */
  float p_rad_edge[MAX_POLY]; /* individual edge radius for polyhedron */
  char p_col_edge[MAX_POLY][40]; /* color of poly edges */
  char col_edge[40];          /* default color and properties of polyhedron edges */

/* plane parameters */

  int nplane;                      /* number of different types of planes */
  int plane_fn[MAX_PLANE];         /* frame number for plane commands */
  char plane_col[MAX_PLANE][40];   /* plane colors */
  char plane_l[MAX_PLANE][5];      /* plane center label */
  float plane_size[MAX_PLANE];     /* size of plane coordination */

/* bond parameters */

  int nbond;                       /* number of different types of bonds */
  int bond_fn[MAX_BOND];           /* frame number for bond commands */
  char bond_l1[MAX_BOND][5];       /* label for one end of bond */
  char bond_l2[MAX_BOND][5];       /* label for other end */
  float bond_size[MAX_BOND];       /* bond diameter */
  float bond_min[MAX_BOND];        /* minimum bond distance */
  float bond_max[MAX_BOND];        /* maximum bond distance */
  char col_bond[MAX_BOND][40];     /* color of bonds */
  int bond_style[MAX_BOND];	   /* solid bond or dash */
  float Bond_Mult;                 /* Multiplier for Bond Size */
  float BndMult;                   /* Bond multiplier initialization */
  float mol_d;                     /* intramolecular distance for molecular and polyhedral completion */

/* text label parameters */

  int nlabel;                        /* number of textual labels */
  float label_x[MAX_LABELS][3];	     /* position of textual labels */
  char label_label[MAX_LABELS][64];  /* label text */
  int label_fn[MAX_LABELS];          /* frame number */
  int labels_moved;
  int autolabel;                     /* flag for autolabel of SHELX atoms */
  int triple[4];

/* least squares plane parameters */

  int nbplane;                       /* number of least squares planes */
  char bplane_t[MAX_BPLANE][10][5];  /* atom labels in plane list */
  int bplane_n[MAX_BPLANE][10];      /* atom numbers in plane list */
  int nbatoms[MAX_BPLANE];           /* number of items in list */
  float bplane_d1[MAX_PLANE];        /* width of plane segment to draw */
  float bplane_d2[MAX_PLANE];        /* second dimension */
  char bplane_col[MAX_PLANE][40];    /* color */

/* sphere parameters */

  int nsphere;                       /* number of different types of spheres */
  int sphere_fn[MAX_SPHERE];         /* frame number for sphere commands */
  char sphere_l[MAX_SPHERE][5];      /* sphere label */
  int  sphere_n[MAX_SPHERE];         /* atom number - -1 for old behavior */
  float sphere_size[MAX_SPHERE];     /* sphere size */
  char sphere_col[MAX_SPHERE][40];   /* sphere colors */
  float Sphere_Mult;                 /* Multiplier for Sphere Size */
  float SpMult;                      /* initialization for Sphere_Mult */


/* background parameters */

  char col_bg[40];             /* Background color */
  float glback[3];             /* Background RGB values for openGL */

/* ellipsoid parameters */

  int auto_ellipse;
  char ellips_col[MAX_EL][40];   /* ellipsoid colors */
  float ellips_EV[MAX_EL][3][3]; /* Eigenvectors */
  float ellips_RMS[MAX_EL][3];   /* RMS amplitudes */
  char ellips_l[MAX_EL][5];      /* label for anisotropic atom */
  int ellips_n[MAX_EL];          /* atom number */
  int save_el_number[MAX_EL];    /* save original atom number */
  int ell_type[MAX_EL];          /* form of ellipsoid input */
  float ellips[MAX_EL][6];       /* thermal ellipsoid coefficients */
  float Phong_Value;             /* reflectivity value for Phong highlighting */
  float Phong_Size;              /* Phong highlighting size */
  char Cutout_color[40];         /* color of cutout section */
  int n_ellips;			 /* number of ellipsoids */
  int do_ellipsoids;             /* non-zero if ellipsoids to be drawn */
  int El_Cutout;                 /* True if ellipsoids to have an octant removed */
  char Ellipaxis_color[40];      /* color of principal axes */
  float Ellipsoid_Prob;          /* Fraction of electron dens enclosed by ellipsoid */
  float Ellipsoid_Scale;         /* Scaling needed to get probability */
  float Ellipaxis_width;         /* linewidth of principal axes */

/* lattice constant variables */
  
  float lat_con[6];          /* lattice constants */
  float rec_lat_con[6];      /* reciprocal lattice constants */
  double b_mat[3][3];        /* metric tensor */
  float ginv[3][3];          /* direct space metric tensor */
  float b_inv_mat[3][3];      /* inverse metric tensor */

/* lattice parameters of cutout box */

  float slab_con[6];         /* lattice constants */
  float slab_off[3];         /* offset */
  float slab_rot[3];         /* rotation angles */

/* variables used in drawing */

    int frame_no;
    int max_frame;              /* maximum number of frames */
    float X_Boxlim;             /* X box limit read from command line */
    float Y_Boxlim;             /* Y box limit read from command line */
    float Z_Boxlim;             /* Z box limit read from command line */
    float X_Origin;             /* X origin read from command line */
    float Y_Origin;             /* Y origin read from command line */
    float Z_Origin;             /* Z origin read from command line */
    float lookat_v1[3], lookat_v2[3];/* "lookat" vectors " */
    int noshadow;               /* True if no shadows for POV objects */
    float ambient;              /* POV lighting options */
    float diffuse;
    float specular;
    float roughness;
    float cryst_lim[MAX_FRAME][6];   /* limits of view box in cell coords */
    float clip_lim[MAX_FRAME][6];    /* limits of clip box in cell coords */

/* stereo parameters */

    int Stereo;                 /* TRUE if stereo pair to be generaged */
    int cross_eyed;             /* TRUE if "cross eyed" view */
    float stereo_base;          /* camera separation */

} CRYSTAL;

